import time
import os
import pytest
from niveristand.legacy import NIVeriStand
from niveristand.legacy.NIVeriStand import NIVeriStandException
from tests.testutilities import configutilities

def sleep():
    time.sleep(1)

TEST_ID = 123213

def test_stimulus_api_legacy():

    wks = NIVeriStand.Workspace()
    print("")
    SYSDEFFILE = os.path.join(configutilities.get_autotest_projects_path(),
                              "TestStimulusAPI", "TestStimulusAPI.nivssdf")
    print("Deploying %s" % SYSDEFFILE)
    wks.RunWorkspaceFile(SYSDEFFILE,0,1,60000,"","")

    try:
        #Verify the TEST_ID var on test file.
        test_ID = wks.GetSingleChannelValue("TEST_ID")
        assert(test_ID == TEST_ID), "Deployed wrong test file"

        STIMULUSPROFILES_DIR = os.path.join(configutilities.get_autotest_projects_path(), "TestStimulusAPI")
        AutoStepTestStimulusAPI = os.path.join(STIMULUSPROFILES_DIR, "AutoStepTestStimulusAPI.nivstest")

        stimTest1 = NIVeriStand.Stimulus()
        stimTest2 = NIVeriStand.Stimulus()

        print("Test Reserve and Unreserve")
        stimTest1.ReserveStimulusProfileManager()
        print("Done reserving stimulus test 1")
        with pytest.raises(NIVeriStandException):
            stimTest2.ReserveStimulusProfileManager()
        print("Start unreserving")
        stimTest1.UnreserveStimulusProfileManager()
        stimTest2.ReserveStimulusProfileManager()
        stimTest2.UnreserveStimulusProfileManager()

        print("Run and get test state")
        result = stimTest1.GetStimulusProfileManagerState()
        assert (result == 0), "Test state not expected"

        print("Running Test " + AutoStepTestStimulusAPI)
        stimTest1.RunStimulusProfile(AutoStepTestStimulusAPI,STIMULUSPROFILES_DIR,60000,1,1)
        sleep()
        result = stimTest1.GetStimulusProfileManagerState()
        assert (result == 2), "Test expected to started already"
        print("Test Getting back the file we are running")
        file = stimTest1.GetStimulusProfileFile()
        assert(file == AutoStepTestStimulusAPI), "Test file are not expected"

        print("Wait untill stimulus is done")
        time.sleep(40)
        result = stimTest1.GetStimulusProfileResult()

        result = stimTest1.GetStimulusProfileManagerState()
        assert (result == 0), "Test should have finished by now"

        stimTest1.RunStimulusProfile(AutoStepTestStimulusAPI,STIMULUSPROFILES_DIR,60000,1,1)
        sleep()
        result = stimTest1.GetStimulusProfileManagerState()
        assert (result == 2), "Test expected to started already"

        stimTest1.StopStimulusProfile()
        sleep()
        result = stimTest1.GetStimulusProfileManagerState()
        assert (result == 0), "Test expected to stop"

        print("Test PASSED")
        print("")

    finally:
        wks.StopWorkspaceFile("")
